/*
 * Copyright 2022, TeamDev. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Redistribution and use in source and/or binary forms, with or without
 * modification, must retain the above copyright notice and the following
 * disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

@file:JvmName("Rules")

package io.spine.validation

import io.spine.protodata.TypeName
import io.spine.validation.event.CompositeRuleAdded
import io.spine.validation.event.SimpleRuleAdded

/**
 * Converts this `rule` to an event.
 *
 * @param type the type name of the validated message
 */
internal fun Rule.toEvent(type: TypeName): RuleAdded {
    return if (hasComposite()) {
        CompositeRuleAdded.newBuilder()
            .setType(type)
            .setRule(composite)
            .build()
    } else {
        SimpleRuleAdded.newBuilder()
            .setType(type)
            .setRule(simple)
            .build()
    }
}

/**
 * Creates a [Rule] from this simple rule.
 */
internal fun SimpleRule.wrap(): Rule =
    Rule.newBuilder()
        .setSimple(this)
        .build()

/**
 * Creates a [Rule] from this composite rule.
 */
internal fun CompositeRule.wrap(): Rule =
    Rule.newBuilder()
        .setComposite(this)
        .build()

/**
 * Creates a [Rule] from this message-wide rule.
 */
internal fun MessageWideRule.wrap(): Rule =
    Rule.newBuilder()
        .setMessageWide(this)
        .build()
